/************************************************************************

      PACKAGE:  IMAGE ENCODER/DECODER TOOLS

      FILE:     CJPEGB.C

      AUTHORS:  Craig Watson
                cwatson@nist.gov
                Michael Garris
                mgarris@nist.gov
      DATE:     01/09/2001

#cat: cjpegb - Takes an IHead or raw image pixmap and encodes it using
#cat:        Baseline (Lossy) JPEG, writing the compressed data to file.

*************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <jpegb.h>
#include <intrlv.h>
#include <ihead.h>
#include <img_io.h>
#include <parsargs.h>

void procargs(int, char **, int *, char **, char **, int *,
              int *, int *, int *, int *, int *, char **);
void print_usage(char *);

int debug = 0;

/******************/
/*Start of Program*/
/******************/

main(int argc, char *argv[])
{
   int ret, i, n;
   int quality, rawflag, intrlvflag;
   char *outext;                        /* ouput file extension */
   char *ifile, *cfile, ofile[MAXPATHLEN]; /* file names */
   FILE *outfp;                         /* output file pointer */
   int width, height, depth, ppi;       /* image parameters */
   int ilen, olen;
   IHEAD *ihead;
   unsigned char *idata, *odata;        /* image pointers */
   char *comment_text;
   int hor_sampfctr[MAX_CMPNTS], vrt_sampfctr[MAX_CMPNTS];

   procargs(argc, argv, &quality, &outext, &ifile, &rawflag,
            &width, &height, &depth, &ppi, &intrlvflag, &cfile);

   if(ret = read_raw_or_ihead(!rawflag, ifile,
                              &ihead, &idata, &width, &height, &depth)){
      exit(ret);
   }

   if(debug > 0)
      fprintf(stdout, "File %s read\n", ifile);

   /* If IHead image file ... */
   if(!rawflag){
      /* Get PPI from IHead. */
      ppi = get_density(ihead);
      free(ihead);
   }

   if(cfile == (char *)NULL)
      comment_text = (char *)NULL;
   else{
      if(ret = read_ascii_file(cfile, &comment_text)){
         free(idata);
         exit(ret);
      }
   }

   /* If necessary, convert to interleaved. */
   if((depth == 24) && (!intrlvflag)){
      for(i = 0; i < MAX_CMPNTS; i++){
         hor_sampfctr[i] = 1;
         vrt_sampfctr[i] = 1;
      }
      if(ret = not2intrlv_mem(&odata, &olen, idata, width, height, depth,
                              hor_sampfctr, vrt_sampfctr, depth>>3)){
         free(idata);
         if(comment_text != (char *)NULL)
            free(comment_text);
         exit(ret);
      }
      free(idata);
      idata = odata;
      ilen = olen;
   }

   if(ret = jpegb_encode_mem(&odata, &olen, quality,
                             idata, width, height, depth, ppi, comment_text)){
      free(idata);
      if(comment_text != (char *)NULL)
         free(comment_text);
      return(ret);
   }

   free(idata);
   if(comment_text != (char *)NULL)
      free(comment_text);

   if(debug > 0)
      fprintf(stdout, "Image data encoded, compressed byte length = %d\n",
              olen);

   fileroot(ifile);
   sprintf(ofile, "%s.%s", ifile, outext);

   if(ret = write_raw_from_memsize(ofile, odata, olen)){
      free(odata);
      exit(ret);
   }

   if(debug > 0)
      fprintf(stdout, "Image data written to file %s\n", ofile);

   free(odata);

   exit(0);
}

/*****************************************************************/
void procargs(int argc, char **argv, int *quality,
              char **outext, char **ifile, int *rawflag,
              int *width, int *height, int *depth, int *ppi,
              int *intrlvflag, char **cfile)
{
   int argi;

   if((argc < 4) || (argc > 8)){
      print_usage(argv[0]);
      exit(-1);
   }

   *quality = atoi(argv[1]);
   *outext = argv[2];
   *ifile = argv[3];
   *rawflag = 0;
   *intrlvflag = 1;
   *width = -1;
   *height = -1;
   *depth = -1;
   *ppi = -1;
   *cfile = (char *)NULL;

   /* If argc == 4, we are done here. */

   /* If IHead image file ... */
   argi = 4;
   if(argi >= argc)
      return;
   if(argc == 5){
      *cfile = argv[argi];
      return;
   }
   /* Otherwise (argc >= 6) expect raw flag ... */
   if(strncmp(argv[argi], "-r", 2) == 0){
      argi++;
      if(argi >= argc){
         print_usage(argv[0]);
         fprintf(stderr,
                 "       option \"-raw_in\" missing \"w,h,d,[ppi]\"\n");
         exit(-1);
      }
      *rawflag = 1;
      parse_w_h_d_ppi(argv[argi], argv[0], width, height, depth, ppi);
      argi++;
      if(argi >= argc)
         return;
   }
   else{
      print_usage(argv[0]);
      fprintf(stderr,
              "       expected \"-raw_in\" option\n");
      exit(-1);
   }

   /* If nonintrlv flag ... */
   if(strncmp(argv[argi], "-n", 2) == 0){
      *intrlvflag = 0;
      argi++;
      if(argi >= argc)
         return;
   }

   /* If we get here, set comment file ... */
   *cfile = argv[argi];
   return;
}

/*****************************************************************/
void print_usage(char *arg0)
{
   fprintf(stderr, "Usage: %s ", arg0);
   fprintf(stderr, "<q=20-95> <outext> <image file>\n");
   fprintf(stderr, "                  [-raw_in w,h,d,[ppi]\n");
   fprintf(stderr, "                     [-nonintrlv]]\n");
   fprintf(stderr, "                  [comment file]\n");
}
