/***********************************************************************
      LIBRARY: IMAGE - Image Manipulation and Processing Routines

      FILE:    WRITIHDR.C

      AUTHOR:  Michael Garris
      DATE:    04/26/1989

      Contains routines responsible for encoding/writing an
      IHead image file.

      ROUTINES:
#cat: writeihdrfile - writes the contents of an IHead structure and an
#cat:                 image memory to the specified file.
#cat: writeihdrsubimage - generates an IHead based on the parameters passed
#cat:                     and writes a subimage to the specified file.
#cat: write_fields - takes a list of IHead structures and binary images
#cat:                and writes them to individual files with the specified
#cat:                root filename and corresponding extension name.

***********************************************************************/
#include <stdio.h>
#include <sys/param.h>
#include <ihead.h>
#include <util.h>

/************************************************************/
/*         Routine:   Writeihdrfile()                       */
/*         Author:    Michael D. Garris                     */
/*         Date:      4/26/89                               */
/*         Modifications:                                   */
/*           9/20/90   (Stan Janet) check return codes      */
/*           2/20/91   (MDG) compression capability         */
/*   Modified 12/94 by Patrick Grother                         */
/*         Allocate 4 times the original image for the rare    */
/*         case (noisy image) where the compressed result has  */
/*         grown in size to be larger than the original image  */

/************************************************************/
/************************************************************/
/* Writeihdrfile() writes a ihead structure and correspon-  */
/* ding image data to the specified file.                   */
/************************************************************/
void writeihdrfile(file,head,data)
char *file;
IHEAD *head;
unsigned char *data;
{
   FILE *fp;
   int width,height,depth,code,filesize,n, compbytes;
   unsigned char *compdata;
   int pred;

   /* reopen the image file for writing */
   fp = fopen(file,"wb");
   if (fp == NULL)
      syserr("writeihdrfile","fopen",file);

   n = sscanf(head->width,"%d",&width);
   if (n != 1)
      fatalerr("writeihdrfile","sscanf failed on width field",NULL);
   n = sscanf(head->height,"%d",&height);
   if (n != 1)
      fatalerr("writeihdrfile","sscanf failed on height field",NULL);
   n = sscanf(head->depth,"%d",&depth);
   if (n != 1)
      fatalerr("writeihdrfile","sscanf failed on depth field",NULL);
   n = sscanf(head->compress, "%d", &code);
   if (n != 1)
      fatalerr("writeihdrfile","sscanf failed on compression code field",NULL);

   filesize = SizeFromDepth(width,height,depth);
   if ( code == CCITT_G4 )  /* G4 compression breaks for noisy images since */
      filesize *= 4;        /* the result is larger than the original. The  */
                            /* buffer is exceeded and a segmentation fault  */
                            /* occurs. The factor 4 is empirically found    */
                            /* to be sufficient.                            */

   if(code == UNCOMP){
      sprintf(head->complen, "%d", 0);
      writeihdr(fp,head);
      n = fwrite(data,1,filesize,fp);
      if (n != filesize)
         syserr("writeihdrfile", "fwrite", file);
   }
   else{
      malloc_uchar(&compdata, filesize, "writeihdrfile : compdata");
      switch(code){
      case CCITT_G3:
         fatalerr("writeihdrfile","G3 compression not implemented.",NULL);
         break;
      case CCITT_G4:
         if(depth != 1)
            fatalerr("writeihdrfile",
                     "G4 compression requires a binary image.", NULL);
         grp4comp(data, filesize, width, height, compdata, &compbytes);
         break;
      case RL:
         rlcomp(data, filesize, compdata, &compbytes, filesize);
         break;
      default:
         fatalerr("writeihdrfile","Unknown compression",NULL);
         break;
      }
      sprintf(head->complen, "%d", compbytes);
      writeihdr(fp,head);
      n = fwrite(compdata,1,compbytes,fp);
      if (n != compbytes)
         syserr("writeihdrfile", "fwrite", file);
      free(compdata);
   }

   /* closes the new file */
   (void) fclose(fp);
}

/************************************************************/
/*         Routine:   Writeihdrsubimage()                   */
/*         Author:    Michael D. Garris                     */
/*         Date:      5/29/91                               */
/************************************************************/
/************************************************************/
/* Writeihdrsubimage() writes a subimage raster along with  */
/* a modified ihead structure to the specified file.        */
/************************************************************/
writeihdrsubimage(name,data,w,h,d,parent,par_x,par_y)
char *name;
unsigned char *data;
int w, h, d;
char *parent;
int par_x, par_y;
{
   IHEAD *ihead;

   if((ihead = (IHEAD *)malloc(sizeof(IHEAD))) == NULL)
      syserr("writeihdrsubimage", "malloc", "ihead");
   nullihdr(ihead);
   strcpy(ihead->id, name);
   strcpy(ihead->created,current_time());
   sprintf(ihead->width, "%d", w);
   sprintf(ihead->height, "%d", h);
   sprintf(ihead->depth, "%d", d);
   sprintf(ihead->compress, "%d", CCITT_G4);
   sprintf(ihead->align, "%d", 8);
   sprintf(ihead->unitsize, "%d", 8);
   ihead->sigbit = MSBF;
   ihead->byte_order = HILOW;
   sprintf(ihead->pix_offset, "%d", 0);
   sprintf(ihead->whitepix, "%d", 0);
   ihead->issigned = UNSIGNED;
   ihead->rm_cm = ROW_MAJ;
   ihead->tb_bt = TOP2BOT;
   ihead->lr_rl = LEFT2RIGHT;
   strcpy(ihead->parent, parent);
   sprintf(ihead->par_x, "%d", par_x);
   sprintf(ihead->par_y, "%d", par_y);
   writeihdrfile(ihead->id, ihead, data);
   free(ihead);
}

/************************************************************/
write_fields(ofile, name_list, num_names, heads, fields, count, compression)
char *ofile, **name_list;
int num_names;
IHEAD **heads;
int count, compression;
unsigned char **fields;
{
  char outname[MAXPATHLEN], tail[MAXPATHLEN];
  int i;

  if(num_names != count)
     fatalerr("write_fields",
              "Table_A name list not equal to number of fields", NULL);
  for(i = 0; i < count; i++){
     sprintf(outname,"%s.%s", ofile, name_list[i]);
     strcpy(tail, outname);
     filetail(tail);
     set_id(heads[i], tail);
     set_compression(heads[i], compression);
     writeihdrfile(outname, heads[i], fields[i]);
   }
}
