/***********************************************************************
      LIBRARY: IOUTIL - INPUT/OUTPUT Utilities

      FILE:    NEWEXT.C
      AUTHOR:  Michael Garris
      DATE:    11/27/89

      Contains routines responsible for replacing a file name's
      extension with a new one.

      ROUTINES:
#cat: newext - takes a filename, strips off the rightmost extenstion, and
#cat:          appends a new extension.  This routine exits upon error.
#cat: newext_ret - takes a filename, strips off the rightmost extenstion, and
#cat:          appends a new extension.  This routine returns an error code.
#cat: newextlong - takes a pointer to a filename, strips the rightmost
#cat:		 extenstion, and appends an arbitrary new extension.

***********************************************************************/

#include <stdio.h>

/*************************************************************/
/* Newext() is a destructive procedure which takes a filename*/
/* and strips off the rightmost extension (if one exists) and*/
/* appends the extension passed.  Exits upon error.          */
/*************************************************************/
newext(char *file, const int len, char *ext)
{
   char *cptr;

   cptr = file + strlen(file);
   while((cptr != file) && (*cptr != '.'))
      cptr--;
   if(cptr == file){
      if(strlen(file) == len)
         fatalerr("newext","File manipulation exceeds allocated memory",NULL);
      cptr += strlen(file);
      *cptr++ = '.';
      /* EDIT MDG 1/25/99
      *cptr = NULL;
      */
      *cptr = '\0';
   }
   else{
      cptr++;
      /* EDIT MDG 1/25/99
      *cptr = NULL;
      */
      *cptr = '\0';
   }
   if(strlen(file) + strlen(ext) > len)
      fatalerr("newext", file, "proposed extension too long");

   strcat(file,ext);
}

/*************************************************************/
/* Newext_ret() is a destructive procedure, taking a filename*/
/* and striping off the rightmost extension (if one exists)  */
/* & appends the extension passed.  Returns code upon error. */
/*************************************************************/
int newext_ret(char *file, int len, char *ext)
{
   char *cptr;

   cptr = file + strlen(file);
   while((cptr != file) && (*cptr != '.'))
      cptr--;
   if(cptr == file){
      if(strlen(file) == len){
         fprintf(
         stderr, "ERROR : newext_ret: file manipulation exceeds memory\n");
         return(-2);
      }
      cptr += strlen(file);
      *cptr++ = '.';
      *cptr = (char)NULL;
   }
   else{
      cptr++;
      *cptr = (char)NULL;
   }
   if(strlen(file) + strlen(ext) > len){
      fprintf(stderr, "ERROR : newext_ret : proposed extension too long\n");
      return(-3);
   }

   strcat(file,ext);

   return(0);
}

/*************************************************************/
newextlong(char **file, char *ext)
{
int n, m;
char *cptr, *tmp;

   n = strlen(*file);      /* the original length                     */
   m = strlen(ext);

   cptr = *file + n;
   while((cptr != *file) && (*cptr != '.'))
      cptr--;

   if (cptr == *file)      /* add an extension where there wasn't one */
   {
      n += m;
      if ((tmp = (char *)calloc(n+1, sizeof(char))) == NULL)
         syserr("newextlong", "calloc", "space for new string result");
      sprintf(tmp, "%s.%s", *file, ext);
      free(*file);
      *file = tmp;
   }
   else
   {                      /* replace the existing extension          */
      *cptr++;
      /* EDIT MDG 1/25/99
      *cptr = NULL;
      */
      *cptr = '\0';
      if ((cptr - *file) + m > n)
      {
         n += m;
         if ((tmp = (char *)calloc(n+1, sizeof(char))) == NULL)
            syserr("newextlong", "calloc", "space for new string result");
         sprintf(tmp, "%s%s", *file, ext);
         free(*file);
         *file = tmp;
      }
      else
         strcat(*file, ext);
   }
}
